import sys
import time
from PIL import Image, ImageFont, ImageSequence
from .ascii import fitting_pixel
from .tty import write_lines, move_up
from .weight import DEFAULT_WEIGHTS


def extract_frames(file_name, fill_empty=False):
    im = Image.open(file_name)
    for frame in ImageSequence.Iterator(im):
        if fill_empty:
            canvas = Image.new("RGBA", frame.size, (0xFF, 0xFF, 0xFF, 0xFF))
            canvas.paste(frame)
            yield canvas
        else:
            yield frame


def _convert_frame(frame, width, height, weights):
    resize_frame = frame.resize((width, height), Image.BICUBIC)
    gray_frame = resize_frame.convert("L")
    all_pixels = gray_frame.load()

    ascii_frame = []
    for y in range(height):
        line = []
        for x in range(width):
            line.append(fitting_pixel(all_pixels[x, y], weights))
        line.append("\n")
        ascii_frame.append("".join(line))
    return ascii_frame


def tty_frame(frame, width, height, weights=DEFAULT_WEIGHTS, backspace=True):
    ascii_frame = _convert_frame(frame, width, height, weights)
    write_lines(ascii_frame)
    if backspace:
        move_up(len(ascii_frame))


def tty_gif(file_name, width, height, sleep, fill_empty=False):
    for frame in extract_frames(file_name, fill_empty):
        tty_frame(frame, width, height)
        time.sleep(sleep)



import functools


def cache(key):
    def decorator(func):
        _cache = {}

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            cache_key = key(args)
            result = _cache.get(cache_key, None)

            if result is None:
                result = func(*args, **kwargs)
                _cache[cache_key] = result
            return result

        return wrapper

    return decorator


@cache(key=lambda args: args[0])
def fitting_pixel(pixel, weights):
    w = float(pixel) / 255
    wf = -1.0
    k = -1
    for i in range(len(weights)):
        if abs(weights[i] - w) <= abs(wf - w):
            wf = weights[i]
            k = i
    return chr(k + 32)



from .gif import extract_frames, tty_frame, tty_gif


def main():
    import argparse

    parser = argparse.ArgumentParser(description="show gif in console mode")
    parser.add_argument("file", help="gif file name")
    parser.add_argument("--width", help="width", type=int, default=60)
    parser.add_argument("--height", help="height", type=int, default=30)
    parser.add_argument("--sleep", help="sleep time", type=float, default=0.02)
    parser.add_argument("--fill", help="fill empty", action="store_true")
    args = parser.parse_args()
    try:
        tty_gif(
            args.file,
            width=args.width,
            height=args.height,
            sleep=args.sleep,
            fill_empty=args.fill,
        )
    except Exception as e:
        print(e)


__all__ = [extract_frames, tty_frame, tty_gif, main]



from PIL import ImageFont


# this function is base on
# http://code.activestate.com/recipes/580702-image-to-ascii-art-converter/
def _default_weights():
    font = ImageFont.load_default()
    (chr_x, chr_y) = font.getsize(chr(32))
    weights = []
    for i in range(32, 127):
        chr_image = font.getmask(chr(i))
        ctr = 0
        for y in range(chr_y):
            for x in range(chr_x):
                if chr_image.getpixel((x, y)) > 0:
                    ctr += 1
        weights.append(float(ctr) / (chr_x * chr_y))
    return weights


DEFAULT_WEIGHTS = _default_weights()
